# Copyright (c) 2016 Nathan Currier

# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

require 'dizby/distributed/proxy'
require 'dizby/access/insecure'

module Dizby
  class DistributedObject
    def self._load(str)
      SemiObjectProxy.new(*Marshal.load(str))
    end

    def _dump(_)
      Marshal.dump [@server.uri, @server.to_id(@obj)]
    end

    def initialize(obj, server)
      @obj = obj
      @server = server
    end

    # rubocop:disable Style/MethodMissing
    def method_missing(msg_id, *args, &block)
      @server.log.debug("calling: #{msg_id} #{args.join ', '}")
      Dizby.check_insecure_method(@obj, msg_id)
      @obj.__send__(msg_id, *args, &block)
    end
    # rubocop:enable Style/MethodMissing

    def respond_to?(msg_id, priv = false)
      responds =
        case msg_id
        when :_dump
          true
        when :marshal_dump
          false
        else
          method_missing(:respond_to?, msg_id, priv)
        end

      @server.log.debug("respond_to?(#{msg_id}) => #{responds}")
      responds
    end

    # rubocop:disable Style/Alias
    alias_method(:respond_to_missing?, :respond_to?)
    # rubocop:enable Style/Alias

    undef :to_s
  end
end
